# last modified 2025-07-29 by J. Fox

import(carData)

importFrom(lattice, barchart, current.panel.limits, densityplot, larrows, llines, lpoints, ltext,
           panel.abline, panel.barchart, #panel.grid,
           panel.polygon, panel.text, strip.custom, strip.default, trellis.par.get,
           trellis.par.set, xyplot)
importFrom(colorspace, rainbow_hcl, sequential_hcl)
importFrom(grid, grid.pretty, grid.segments, unit)
importFrom(lme4, fixef)
importFrom(nnet, multinom)
importFrom(graphics, plot)
importFrom(grDevices, gray, palette, rgb)
importFrom(survey, svymean)
importFrom(insight, find_formula,get_parameters,get_varcov)
importFrom(stats, as.formula, binomial, coef, coefficients, cov, delete.response, family, fitted,
    formula, glm, glm.control, lm, lm.fit, loess.smooth, mahalanobis, make.link, median,
    model.frame, model.matrix, model.offset,
    model.response, na.exclude, na.omit, nlm, predict, qf, qnorm, qt, quantile, residuals,
    spline, terms, update, vcov, weights, xtabs)
importFrom(utils, menu, browseURL)

export(effect, allEffects, Effect, effectsTheme)
export(predictorEffect, predictorEffects)
export(effectsHexsticker)
export(effCoef)
export(levels2dates)
S3method(plot, predictoreff)
S3method(plot, predictorefflist)
S3method(predictorEffects, default)
S3method(predictorEffects, poLCA)
S3method(predictorEffect, default)
S3method(predictorEffect, svyglm)
S3method(predictorEffect, poLCA)
S3method(effCoef, default)

S3method(Effect, default)
S3method(Effect, lm)
S3method(Effect, poLCA)
S3method(Effect, mlm)
S3method(Effect, svyglm)
S3method(Effect, multinom)
S3method(Effect, polr)
S3method(Effect, merMod)

export(predictorEffect.default,predictorEffects.default, 
       Effect.default, effect.default, allEffects.default)
S3method(effSources,betareg)
S3method(effSources,clm)
S3method(effSources,clm2) 
S3method(effSources,clmm)
S3method(effSources,default) 
S3method(effSources,glmmPQL)
S3method(effSources,gls)
S3method(effSources,rlmerMod)
export(effSources,effSources.default)
#,effSources.gls,effSources.glmmPQL,
#       effSources.rlmerMod,effSources.clm,effSources.clm2,
#       effSources.clmm,effSources.betareg)

S3method(print, eff)
S3method(print, efflist)
S3method(print, mlm.efflist)
S3method(print, summary.eff)
S3method(print, predictoreff)
S3method(print, predictorefflist)
S3method(summary, eff)
S3method(summary, efflist)
S3method(summary, mlm.efflist)
S3method(summary, predictorefflist)
S3method(as.data.frame, eff)
S3method(as.data.frame, efflist)
S3method(as.data.frame, effpoly)
S3method(as.data.frame, efflatent)
S3method(plot, eff)
S3method(print, plot.eff)
S3method(plot, efflist)
S3method(plot, mlm.efflist)
S3method(print, effpoly)
S3method(summary, effpoly)
S3method(plot, effpoly)
S3method(print, efflatent)
S3method(summary, efflatent)
S3method(allEffects, default)
S3method(allEffects, poLCA)
S3method(allEffects, mlm)
S3method(effect, default)
S3method(vcov, eff)
S3method(`[`, efflist)
S3method(levels2dates, effpoly)
S3method(levels2dates, eff)
